// Example testing sketch for various DHT humidity/temperature sensors
// Written by ladyada, public domain
#include<Arduino.h>
#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>
#include <WiFiClient.h>
ESP8266WiFiMulti wifiMulti;
#include "DHT.h"

#define DHTPIN D4     // what digital pin we're connected to

// Uncomment whatever type you're using!
#define DHTTYPE DHT11   // DHT 11
//#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321
//#define DHTTYPE DHT21   // DHT 21 (AM2301)

DHT dht(DHTPIN, DHTTYPE);
String s;
int h = 0.0;
float t  = 0.0;
int hic  = 0.0;
int sec = 0;
int mins = 0;
int hr = 0;

// pub sub client
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
IPAddress mqttserver(192, 168, 1, 111);

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();
}

WiFiClient ethClient;
PubSubClient mqttClient(ethClient);


void mqttReconnect() {
  // Loop until we're reconnected
  while (!mqttClient.connected()) {
    Serial.print("Attempting MQTT connection...");
    if (mqttClient.connect("arduinoClient")) {
      Serial.println("connected");
      mqttClient.subscribe("inTopic");
    } else {
      Serial.print("failed, rc=");
      Serial.print(mqttClient.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
/*************************************************/
String cat;
String catTemp;
String catHum;
void setup() {

  Serial.begin(115200);
  Serial.println("DHTxx test!");

  dht.begin();

  WiFi.mode(WIFI_STA);
  wifiMulti.addAP("NODE_GROUND", "IAmAMeshNetworkPassword");
  Serial.println(WiFi.macAddress());

  Serial.print("Connecting Wifi...");
  while (wifiMulti.run() != WL_CONNECTED) {
    Serial.print(".");
    delay(1000);
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  // pub sub client
  mqttClient.setServer(mqttserver, 1883);
  mqttClient.setCallback(callback);
  mqttClient.subscribe("inTopic");
  cat = "node/"+String(ESP.getChipId());
  catTemp = cat + "/temp";
  catHum = cat + "/hum";
}


int maxWait = 5000;
int wait = 0;
void loop() {
  // Wait a few seconds between measurements.
  if (wifiMulti.run() != WL_CONNECTED) {
    Serial.println("WiFi not connected!");

  }

  if (!mqttClient.connected()) {
    mqttReconnect();
  }
  mqttClient.loop();

  if (wait < maxWait) { // 
    wait++;
    delay(1);
  } else {


    // Reading temperature or humidity takes about 250 milliseconds!
    // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
    h = dht.readHumidity();
    // Read temperature as Celsius (the default)
    t = dht.readTemperature();

    // Check if any reads failed and exit early (to try again).
    if (isnan(h) || isnan(t) ) {
      Serial.println("Failed to read from DHT sensor!");

    } else {

      Serial.println("send :"+catTemp+String(t)+"*C");
      mqttClient.publish(catTemp.c_str(), (String(t)+"*C").c_str());
      mqttClient.publish(catHum.c_str(), (String(h)+"%").c_str());


    }
    wait = 0;
  }
}
